# "mkinitramfs" command
#
# Copyright (C) 2016  Patrick McDermott
#
# This file is part of the ProteanOS Development Kit.
#
# The ProteanOS Development Kit is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# The ProteanOS Development Kit is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the ProteanOS Development Kit.  If not, see
# <http://www.gnu.org/licenses/>.

cmd_mkinitramfs_optstring='l:i:'

cmd_mkinitramfs_main()
{
	local linux_output=
	local initramfs_output=
	local root=
	local dev=

	if ! get_options "${@}"; then
		print_cmd_usage 'mkinitramfs' >&2
		return 1
	fi
	shift $((${OPTIND} - 1))

	if [ "x${cmd_mkinitramfs_opt_l-}" = 'x' ]; then
		print_cmd_usage 'mkinitramfs' >&2
		return 1
	fi
	if [ "x${cmd_mkinitramfs_opt_i-}" = 'x' ]; then
		print_cmd_usage 'mkinitramfs' >&2
		return 1
	fi
	if [ ${#} -lt 1 ]; then
		print_cmd_usage 'mkinitramfs' >&2
		return 1
	fi

	linux_output="${cmd_mkinitramfs_opt_l}"
	initramfs_output="${cmd_mkinitramfs_opt_i}"
	root="${1}"
	shift 1

	dev=''
	if is_block "${root}"; then
		dev="${root}"
		if ! root="$(block_mount "${dev}")"; then
			return 2
		fi
	fi

	if ! profile_detect "${root}"; then
		[ "x${dev}" != 'x' ] && block_umount "${root}"
		return 2
	fi
	arch="$(cat -- "${root}/etc/proteanos_arch")"
	plat="$(cat -- "${root}/etc/proteanos_plat")"

	if ! img="$(profile_find_kernel "${root}" "${arch}" "${plat}")"; then
		error "$(get_msg 'cmd_mkinitramfs_kernel_not_found')"
		[ "x${dev}" != 'x' ] && block_umount "${root}"
		return 2
	fi
	cp -p -- "${root}/${img}" "${linux_output}"

	(cd -- "${root}" && profile_make_initramfs "${arch}" "${plat}") \
		>"${initramfs_output}"

	[ "x${dev}" != 'x' ] && block_umount "${root}"

	return 0
}

cmd_mkinitramfs_register()
{
	register_cmd 'mkinitramfs'
}
__init cmd_mkinitramfs_register
